#
#  - map manipulating procedures -
#
#   Created:    07/00
#   Changed:    12/01 
#
########################################################################

################################################################
#
# LoadMap --
#
# Load configuration data for a patch map from file
#
# Arguments:     patchname ... patch name (f.e. "A-1")
#
# Returns:    -
#
proc LoadMap {patchname} {
global map

	set filename ${patchname}.cfg

	if {![file exist $filename]} {

		# - Init -
		# at first create new config file for this patch
		LoadMap init
		foreach name [lsort [array name map init,*]] {
			set n [lindex [split $name ","] 1]
			set map($patchname,$n) $map($name)
		}
		set map($patchname,name) "{}"
		set map($patchname,description) "{}"
		set map($patchname,lower_name) "{}"
		set map($patchname,upper_name) "{} "

		set bank [lindex [split $patchname "-"] 0]
		set patch [lindex [split $patchname "-"] 1]
		set addr [expr ($patch - 1) * 16]
		if { $bank == "B" } {
			incr addr 512
		}
		set map($patchname,address) $addr

		SaveMap $patchname
	}

	set f [open $filename r]

	while {![eof $f]} {
		gets $f s
		eval $s
	}

	close $f
	return 1
}

################################################################
#
# SaveMap --
#
# Save configuration data for a patch map into file
#
# Arguments:     patchname ... patch name (f.e. "A-1")
#
# Returns:    -
#
proc SaveMap {patchname} {
global map

	set filename ${patchname}.cfg
	set f [open $filename w]

	foreach name [lsort [array name map $patchname,*]] {
		puts $f "set map($name) $map($name)"
	}
	
	close $f
}

################################################################
#
# Map2Patch --
#
# Converts the universal map data into the current patch for GUI
#
# Arguments:     patchname ... patch name (f.e. "A-1")
#
# Returns:    -
#
proc Map2Patch {patchname} {
global a33 map def

	# Convert the whole map
	foreach name [array name map $patchname,*] {
		set n [lindex [split $name ","] 1]
		set a33($n) $map($name)
	}

	# update display
	UpdateDisplay "PATCH: $a33(oldBank)-$a33(oldPatch)" "$a33(name)"

	# special converts
	set a33(lever) [lindex $def(lever) $a33(lever)]

	set a33(velocity) [lindex $def(velocity) [expr $a33(velocity) - 1]]

	set index [lsearch $def(cc_value) $a33(data_entry)]
	if {$index == -1} {set index 0}
	set a33(_data_entry) [lindex $def(cc_name) $index]
	
	set index [lsearch $def(transpose_value) $a33(transpose)]
	if {$index == -1} {set index 0}
	set a33(_transpose) [lindex $def(transpose_name) $index]

	set index [lsearch $def(octave_value) $a33(lower_octave)]
	if {$index == -1} {set index 0}
	set a33(lower_oct) [lindex $def(octave_name) $index]
	set index [lsearch $def(octave_value) $a33(upper_octave)]
	if {$index == -1} {set index 0}
	set a33(upper_oct) [lindex $def(octave_name) $index]

	incr a33(lower_midichannel)
	incr a33(upper_midichannel)
	incr a33(lower_program)
	incr a33(upper_program)

}

################################################################
#
# Patch2Map --
#
# Converts the current patch from GUI into universal map data 
#
# Arguments:     patchname ... patch name (f.e. "A-1")
#
# Returns:    -
#
proc Patch2Map {patchname } {
global a33 map def

	# Convert the parameter for A33
	foreach name [array name map $patchname,*] {
		set n [lindex [split $name ","] 1]
		set map($name) $a33($n)
	}

	# special converts
	set map($patchname,lever) [lsearch $def(lever) $a33(lever)]

	set map($patchname,velocity) [expr [lsearch $def(velocity) $a33(velocity)] + 1]

	set index [lsearch $def(cc_name) $a33(_data_entry)]
	set map($patchname,data_entry) [lindex $def(cc_value) $index]
	
	set index [lsearch $def(transpose_name) $a33(_transpose)]
	set map($patchname,transpose) [lindex $def(transpose_value) $index]
	set index [lsearch $def(octave_name) $a33(lower_oct)]
	set map($patchname,lower_octave) [lindex $def(octave_value) $index]
	set index [lsearch $def(octave_name) $a33(upper_oct)]
	set map($patchname,upper_octave) [lindex $def(octave_value) $index]

	incr map($patchname,lower_midichannel) -1
	incr map($patchname,upper_midichannel) -1 
	incr map($patchname,lower_program) -1
	incr map($patchname,upper_program) -1

	set map($patchname,name) [list $a33(name)]
	set map($patchname,description) [list $a33(description)]
	set map($patchname,lower_name) [list $a33(lower_name)]
	set map($patchname,upper_name) [list $a33(upper_name)]

}

################################################################
#
# DiffMap --
#
# Check the recorded map saved in the A33 controller against the 
# current map. 
#
# Arguments:   patchname 
#
# Returns:    > 1 ... patches differs in "n" cases
#		  1 ... patches are equal
#
proc DiffMap  {patchname} {
global recmap map

	set deny "name description lower_name upper_name"	

	set diff 1
	set difflist "<$patchname>"

	foreach i [lsort [array names recmap]] {

		set patch [lindex [split $i ","] 0]		
		set param [lindex [split $i ","] 1]		

		# avoid deny list
		if { [lsearch $deny $param] != "-1" } {
			continue
		}

		# create difflist
		if { ( ($patch == $patchname) || ($patchname == "0-0") ) && ($recmap($patch,$param) != $map($patch,$param)) } {
			lappend difflist "$i: $recmap($patch,$param) != $map($patch,$param)"
		}
	}

	# patch maps differs
	if {[llength $difflist] > 1} {
#		foreach i $difflist {puts $i}
		return [llength $difflist]
	} 

	return 1
}


